// ecuflash.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "..\common\ecutools.h"
#include "..\common\log.h"
#include "resource.h"

void usage()
{
	printf(
		"EcuFlash V0.90\n"
		"flashes WRX Denso ECU plus other functions.\n\n"
		"ecuflash {switches}\n"
		"\n"
		"where switches are:\n"
		"\n"

		"   -one of the following switches-\n\n"       
		"   /ci [image file] - compare image\n"
		"   /fi [image file] - flash image\n"
		"   /tfi [image file] - test flash image (no writing)\n"
		"   /dm [hex start] [hex end] [dumpfile] - dump specified memory area to a file\n"
		"   /dq [filebase] - quick dump all of flash memory to file(s)\n\n"
		"   -and any of these optional switches-\n\n"       
//		"   /k [kernel image file] - use a special kernel image\n"
		"   /l [logfile] - log output to a file\n"
//		"	/ll {none,verbose} - set logging level\n"
		"   /c [comport] - select comport for interface hardware\n"
//		"   /m {wrx2002,wrx2004,sti2004,fxt2004}\n"
		"   /m {wrx2002,wrx2004} - select vehicle model\n"
//		"   /s - silent (no user input req'd)\n"
		);
	exit(0);
}

bool silent = false;

bool wait_for_key(char key)
{
	if (silent)
		return true;

	while(!_kbhit());
	
	return (_getch() == key);
}

int _tmain(int argc, _TCHAR* argv[])
{
	log l;
	string imagefile,logfile,comport,dumpfile,dumpfilebase,kernelfile;
	int dumpstart=0,dumpend=0;
	bool nowrite=false;
	bool compare=false;
	ecumodel model = wrx2002;

	comport = "COM1:";

	for (int argi = 1; argi < argc; argi++)
	{
		if (argv[argi][0] == '/' || argv[argi][0] == '-')
		{
			// looks like a switch
			char *sw = &argv[argi][1];
			if (strcmp(sw,"k") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();
				
				kernelfile = argv[argi];
			}
			else if (strcmp(sw,"fi") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				imagefile = argv[argi];
			}
			else if (strcmp(sw,"tfi") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				imagefile = argv[argi];
				nowrite = true;
			}
			else if (strcmp(sw,"ci") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				imagefile = argv[argi];
				compare = true;
			}
			else if (strcmp(sw,"dm") == 0)
			{
				argi++;
				if (argi+2 >= argc)
					usage();
				dumpstart = strtol(argv[argi++],NULL,16);
				dumpend = strtol(argv[argi++],NULL,16);
				dumpstart &= 0xFFFFFFFE;
				dumpend |= 0x00000001;
				dumpfile = argv[argi++];
			}			
			else if (strcmp(sw,"dq") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();
				dumpfilebase = argv[argi++];
			}			
			else if (strcmp(sw,"l") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				logfile = argv[argi];
				l.set_filename(logfile);
			}
			else if (strcmp(sw,"ll") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				if (strcmp(argv[argi],"verbose") == 0)
					l.set_mask(0xFFFFFFFF);
				else if (strcmp(argv[argi],"none") == 0)
					l.set_mask(0x00000000);
				else
					usage();
			}
			else if (strcmp(sw,"c") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				comport = argv[argi];
			}
			else if (strcmp(sw,"m") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				if (strcmp("wrx2002",argv[argi]) == 0)
					model = wrx2002;
				else if (strcmp("wrx2004",argv[argi]) == 0)
					model = wrx2004;
				else
					usage();
			}
			else if (strcmp(sw,"s") == 0)
			{
				silent = true;
			}
			else
				usage();
		}
		else
			usage(); // no non-switch args
	}


	l.set_timestamp(false);
	l.open();
	ecutools ecut(comport);
	ecut.addlog(&l);
	ecut.set_model(model);

	if (!dumpfile.empty())
	{
		if (kernelfile.empty())
		{
			if (!ecut.read_kernel_from_resource(IDR_BINARY1))
				exit(0);
		}
		else if (!ecut.read_kernel(kernelfile))
			exit(0);
		l.print("kernel image loaded and verified.\n");

		if (!ecut.ready_port())
		{
			printf("unable to connect to port.\n");
			exit(0);
		}
		l.print("Connect the green test mode connector under your dash, turn your ignition to the 'ON' position,");
		l.print("then quickly press the D key to dump to ecu memory from %X to %X,\nor any other key to cancel.\n",dumpstart,dumpend);
		if (wait_for_key('d'))
		{
			if (ecut.load_kernel())
			{
				ecut.dump_memory_to_file(dumpfile,dumpstart,dumpend);
			}
		}
	}
	else if (!dumpfilebase.empty())
	{
		if (kernelfile.empty())
		{
			if (!ecut.read_kernel_from_resource(IDR_BINARY1))
				exit(0);
		}
		else if (!ecut.read_kernel(kernelfile))
			exit(0);
		l.print("kernel image loaded and verified.\n");

		if (!ecut.ready_port())
		{
			printf("unable to connect to port.\n");
			exit(0);
		}
		l.print("Connect the green test mode connector under your dash, turn your ignition to the 'ON' position,");
		l.print("then quickly, press the D key to dump all of ecu memory,\nor any other key to cancel.\n");
		if (wait_for_key('d'))
		{
			if (ecut.load_kernel())
			{
				ecut.dump_all_memory_to_file(dumpfilebase);
			}
		}
	}
	else if (!imagefile.empty())
	{	
		if (kernelfile.empty())
		{
			if (!ecut.read_kernel_from_resource(IDR_BINARY1))
				exit(0);
		}
		else if (!ecut.read_kernel(kernelfile))
			exit(0);
		l.print("kernel image loaded and verified.\n");
		if (!ecut.read_image(imagefile))
			exit(0);
		if (!ecut.ready_port())
		{
			printf("unable to connect to port.\n");
			exit(0);
		}
		l.print("ecu image loaded.\n");
		l.print("Connect the green test mode connector under your dash, turn your ignition to the 'ON' position,");
		if (nowrite)
		{
			l.print("Press the T key to test flash (no writing) ecu with this image,\nor any other key to cancel.\n");
			if (wait_for_key('t'))
				ecut.flash_image(false);
		}
		else if (compare)
		{
			l.print("Press the C key to compare ecu with this image,\nor any other key to cancel.\n");
			if (wait_for_key('c'))
				ecut.compare_image();
		}
		else
		{
			l.print("Press the F key to flash ecu with this image,\nor any other key to cancel.\n");
			if (wait_for_key('f'))
				ecut.flash_image(true);
		}
		ecut.close_port();
	}	
	else
		usage();	

	l.close();
}