// ecusim.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "..\common\densoecu.h"
#include "..\common\log.h"

void usage()
{
	printf(
		"emulates WRX Denso ECU.\n\n"
		"ecusim {switches}\n"
		"\n"
		"where switches are:\n"
		"\n"
		"   /ri [rominputfile]\n"
		"   /ro [romoutputfile]\n"
		"   /l [logfile]\n"
		"	/ll {none,verbose} - set logging level\n"
		"   /c [comport]\n"
		"   /q - listen only (don't respond)\n"
		"   /m {wrx2002,wrx2004,sti2004,fxt2004}\n"
		"   /to [seconds] - exit after specified amount of inactivity\n"
		);
	exit(0);
}

int _tmain(int argc, _TCHAR* argv[])
{
	log l;
	string rominfile,romoutfile,logfile,comport;
	bool listenonly = false;
	ecumodel model = wrx2002;
	int timeout = 0;

	comport = "COM1:";

	for (int argi = 1; argi < argc; argi++)
	{
		if (argv[argi][0] == '/' || argv[argi][0] == '-')
		{
			// looks like a switch
			char *sw = &argv[argi][1];
			if (strcmp(sw,"ri") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();
				
				rominfile = argv[argi];
			}
			else if (strcmp(sw,"ro") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				romoutfile = argv[argi];
			}
			else if (strcmp(sw,"l") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				logfile = argv[argi];
				l.set_filename(logfile);
			}
			else if (strcmp(sw,"ll") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				if (strcmp(argv[argi],"verbose") == 0)
					l.set_mask(0xFFFFFFFF);
				else if (strcmp(argv[argi],"none") == 0)
					l.set_mask(0x00000000);
				else
					usage();
			}
			else if (strcmp(sw,"c") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				comport = argv[argi];
			}
			else if (strcmp(sw,"m") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				if (strcmp("wrx2002",argv[argi]) == 0)
					model = wrx2002;
				else if (strcmp("wrx2004",argv[argi]) == 0)
					model = wrx2004;
				else if (strcmp("sti2004",argv[argi]) == 0)
					model = sti2004;
				else if (strcmp("fxt2004",argv[argi]) == 0)
					model = fxt2004;
				else
					usage();
			}
			else if (strcmp(sw,"to") == 0)
			{
				argi++;
				if (argi+1 > argc)
					usage();
			
				sscanf(argv[argi],"%d",&timeout);
			}
			else if (strcmp(sw,"q") == 0)
			{
				listenonly = true;
			}
			else
				usage();
		}
		else
			usage(); // no non-switch args
	}

	l.open();
	densoecu ecu(comport);
	ecu.listenonly(listenonly);
	ecu.set_ecu_sim_mode(true);
	ecu.set_model(model);

	ecu.addlog(&l);

	if (!rominfile.empty())
		ecu.load_rom_from_file(rominfile);

	ecu.open();
	int lastrxtime = (int)time(NULL);
	while(1)
	{
		if (ecu.listen())
			lastrxtime = (int)time(NULL);
		if (timeout != 0 && time(NULL) - lastrxtime > timeout)
			break;
		if (_kbhit())
		{
			if (_getch() == 'q')
				break;
		}
	}
	if (!romoutfile.empty())
		ecu.save_rom_to_file(romoutfile);

	ecu.close();
	l.close();
}