// kerncrypt.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "../common/densoecu.h"

void usage()
{
	printf(
		"encrypts or decrypts a Denso kernel image.\n\n"
		"kerncrypt [infile] [outfile] {switches}\n\n"
		"    [outfile]   binary file to write\n"
		"    /e encrypt (from 68HC16 binary to Denso format)\n"
		"    /d decrypt (from Denso format to 68HC16 binary)\n"
		"    /m {wrx2002,wrx2004,sti2004,fxt2004}\n"
		);
	exit(0);
}

int _tmain(int argc, _TCHAR* argv[])
{
	bool encrypt = true;
	FILE *fpi,*fpo;
	string infile,outfile;
	ecumodel model = wrx2002;

	for (int argi = 1; argi < argc; argi++)
	{
		if (argv[argi][0] == '/' || argv[argi][0] == '-')
		{
			// looks like a switch
			char *sw = &argv[argi][1];
			if (strcmp(sw,"e") == 0)
			{
				encrypt = true;
			}
			else if (strcmp(sw,"d") == 0)
			{
				encrypt = false;
			} 
			else if (strcmp(sw,"m") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				if (strcmp("wrx2002",argv[argi]) == 0)
					model = wrx2002;
				else if (strcmp("wrx2004",argv[argi]) == 0)
					model = wrx2004;
				else if (strcmp("sti2004",argv[argi]) == 0)
					model = sti2004;
				else if (strcmp("fxt2004",argv[argi]) == 0)
					model = fxt2004;
				else
					usage();
			}
			else
				usage();
		}
		else
		{
			if (infile.empty())
				infile = argv[argi];
			else if (outfile.empty())
				outfile = argv[argi];
			else
				usage();
		}
	}
	if (infile.empty() || outfile.empty())
		usage();

	if (NULL == (fpi = fopen(infile.c_str(),"rb")))
	{
		printf("can't open input file.\n");
		return 0;
	}
	if (NULL == (fpo = fopen(outfile.c_str(),"wb")))
	{
		printf("can't open output file.\n");
		return 0;
	}
	
	static const int blksz = 1024;
	unsigned char block[blksz];
	int bytesread;
	int offset = 0;

	while (0 != (bytesread = (int)fread(block,1,blksz,fpi)))
	{
		switch(model)
		{
		case wrx2002:
			densoecu::transform_kernel_block02(block,bytesread,offset,encrypt);
			break;
		case wrx2004:
			densoecu::transform_kernel_block04(block,bytesread,encrypt);
			break;
		default:
			// not supported yet
			printf("model not supported.\n");
			exit(0);
			break;
		}
		if (0 == fwrite(block,1,bytesread,fpo))
		{
			printf("failed writing output file.\n");
			return 0;
		}
		offset += bytesread;
	}
	fclose(fpo);
	fclose(fpi);
}