// optest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "..\common\serial.h"

#define TESTBLKSZ 4096

void usage()
{
	printf(
		"tests OpenPort functionality.\n\n"
		"optest {switches}\n"
		"\n"
		"where switches are:\n"
		"\n"
		"   /c [comport]\n"
		"   /b [baudrate]\n"
		"   /f - determines fastest reliable baud rate\n"
		"   /l [loops]\n"
		);
	exit(0);
}

bool silent = false;

bool wait_for_key(char key)
{
	if (silent)
		return true;

	while(!_kbhit());
	
	return (_getch() == key);
}

int _tmain(int argc, _TCHAR* argv[])
{
	string comport;
	bool flashstock = false;
	int baudrate = 0;
	bool findfastest = false;
	int loops = 1;
	int i,j;
	bool error = false;

	comport = "COM1:";
	baudrate = 38400;
	int lobaudrate = 38400;
	int hibaudrate = 115200;

	for (int argi = 1; argi < argc; argi++)
	{
		if (argv[argi][0] == '/' || argv[argi][0] == '-')
		{
			// looks like a switch
			char *sw = &argv[argi][1];

			if (strcmp(sw,"c") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				comport = argv[argi];
			}					
			else if (strcmp(sw,"b") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				sscanf(argv[argi],"%d",&baudrate);
			}
			else if (strcmp(sw,"l") == 0)
			{
				argi++;
				if (argi >= argc)
					usage();

				sscanf(argv[argi],"%d",&loops);
			}
			else if (strcmp(sw,"f") == 0)
			{
				findfastest = true;
			}
			else
				usage();
		}
		else
			usage(); // no non-switch args
	}

	serial s;
	s.set_port(comport);
	s.set_baud(baudrate);
	if (!s.open())
	{
		printf("unable to open port.");
		exit(0);
	}
	s.flush();
	Sleep(500);


	printf("testing L-line.\n",baudrate);
	error = false;
	for (i = 0; i < 100; i++)
	{
		if (i % 2)
		{
			printf(".");
			s.set_status(SETDTR);
			Sleep(20);
			if (!(s.get_status() & MS_DSR_ON))
			{
				printf("DSR low when should be high.\n");
				error = true;
				break;
			}
		}
		else
		{
			printf("*");
			s.set_status(CLRDTR);
			Sleep(20);
			if (s.get_status() & MS_DSR_ON)
			{
				printf("DSR high when should be low.\n");
				error = true;
				break;
			}
		}
	}
	if (!error)
		printf("L-line OK!");

	unsigned char datatx[TESTBLKSZ];
	unsigned char datarx[TESTBLKSZ];


		for (i = 0; i < loops; i++)
		{

			printf("testing K-line at %d baud.\n",baudrate);
			
			for (int h = 0; h < 10; h++)
			{
				for (j = 0; j < TESTBLKSZ; j++)
					datatx[j] = rand() % 256;
				printf(".");
				s.write(datatx,TESTBLKSZ);
				if (!s.read(datarx,TESTBLKSZ))
				{
					printf("missing response.\n");
					exit(0);
				}
				if (memcmp(datatx,datarx,TESTBLKSZ) != 0)
				{
					printf("response does not match.\n");
					int k;
					for (k = 0; k < TESTBLKSZ; k++)
					{
						if (datatx[k] != datarx[k])
						{
							printf("at position %d, sent %02X, received %02X.\n",k,datatx[k],datarx[k]);
							break;
						}
					}
					exit(0);
					hibaudrate = baudrate;
					break;
				}
				else
				{
					if (h == 99)
						lobaudrate = baudrate;
				}
			}
			if (findfastest)
			{
				baudrate = (int)(baudrate * 1.05);
				s.set_baud(baudrate);
				unsigned char x = 0x5A;
				s.write(&x,1);
				Sleep(50);
				s.flush();
			}
		}

	printf("K-line OK!");

	exit(1);
}
